function I=IsolMatrix(SM,Mplusfun)
% IsolMatrix  Computes a single fault isolability matrix
%
%    I=IsolMatrix(SM[,Mplusfun])
%
%  Inputs:
%    SM       - A structural model object
%    Mplusfun - A function handle to a function computing M+, 
%               defaults to @computeMixed. See examples below how
%               to compute an isolability matrix for mixed, derivative, and
%               integral causality respectively.
%
%  Output:
%   I         - An isolability matrix, A 1 in position (i,j) indicates that
%               fault j is not structurally, under the given causal
%               interpretation, isolable from fault i.
%
%  Example:
%     Imix=IsolMatrix(SM,@computeMixed); % Mixed causality
%     Ider=IsolMatrix(SM,@computeDeriv); % Derivative causality
%     Iint=IsolMatrix(SM,@computeInteg); % Integral causality

% Author(s): Erik Frisk, Mattias Krysander
% Revision: 0.1, Date: 2010/09/12

% Copyright (C) 2010 Erik Frisk and Mattias Krysander
%
% This file is part of CausalIsolability.
% 
% CausalIsolability is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
% 
% CausalIsolability is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%   
% You should have received a copy of the GNU General Public License
% along with CausalIsolability; if not, write to the Free Software
% Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  if nargin < 2
    Mplusfun=@computeMixed
  end

  nf = length(SM.f);
  I=ones(nf,nf);
  
  for f1=1:nf
    % Decouple fault f1
    f1eqs=find(SM.F(:,f1)==0);
    X = SM.X(f1eqs,:);
    G = Mplusfun(X);
    I(f1,find(any(SM.F(f1eqs(G.row),:),1)))=0;
  end
end

    
